""" Main file """
import logging
from typing import Union
from .sport80_http_client import SportEightyHTTP


class SportEighty:
    """
    This class enables a variety of functions that can be carried out with a sport80 subdomain.
    """
    def __init__(self, subdomain: str, return_dict=True, debug: logging = logging.WARNING):
        self.__http_client = SportEightyHTTP(subdomain, return_dict=return_dict, debug_lvl=debug)

    def event_index(self, year: int) -> dict:
        """ Now working """
        return self.__http_client.get_event_index(year)

    def event_results(self, event_dict: dict) -> Union[list, dict]:
        """ Now working """
        return self.__http_client.get_event_results(event_dict)

    def upcoming_events(self) -> Union[list, dict]:
        """ Now working """
        return self.__http_client.get_upcoming_events()

    def start_list(self, event_id: str) -> Union[list, dict]:
        """ Now working """
        return self.__http_client.get_start_list(event_id)

    def rankings_index(self) -> dict:
        """ Returns a dict containing endpoints for all available ranking categories """
        return self.__http_client.get_ranking_index()



"""
Totally unofficial Sport80 API package. Meh.
"""

from .sport80 import SportEighty
from .sport80_http_client import SportEightyHTTP
from .helpers import pull_tables, convert_to_json, convert_to_py
from .pages_enum import EndPoint

__version__ = "2.2.2"
__author__ = "Euan Meston"



""" Throw all the endpoint strings in here """
import enum


class LegacyEndPoint(enum.Enum):
    """ As mentioned before... """
    EVENT_INDEX = "event_results?id_ranking=8"
    EVENT_RESULTS = f"{EVENT_INDEX}&resource="
    UPCOMING_EVENTS = "events"
    START_LIST = "public_reports/index/"
    IS_LOGGED_IN = "register/user_status?id_ranking=8"


class EndPoint(enum.Enum):
    """ This is for the OpenAPI endpoints that the latest Sport80 pages are using """
    ORGANISATIONS = "/api/organisations"
    EVENT_INDEX = "/api/events/table/data"
    INDEX_PAGE = "/public/rankings/"
    ALL_RANKINGS = "/api/categories/all/rankings/table/data"
    RANKINGS_INDEX = "/api/categories/featured"
    RANKINGS_DATA = "/api/categories/rankings"

    @staticmethod
    def event_results_url(event_id) -> str:
        """ Simple method for creating the correct API call """
        api_url = f"/api/events/{event_id}/table/data"
        return api_url

    @staticmethod
    def lifter_url(lifter_id) -> str:
        """ Simple method for creating the correct API call """
        api_url = f"/api/athletes/{lifter_id}/table/data"
        return api_url

    @staticmethod
    def rankings_url(category_id) -> str:
        """ Simple method for creating the correct API call """
        api_url = f"/api/categories/{category_id}/rankings/table"
        return api_url



"""
Helpers library of static functions
"""
import csv
import re
import socket
from logging import info, debug
from bs4 import BeautifulSoup
from requests import Response
from js2py import eval_js

from .pages_enum import LegacyEndPoint


def __single_dict_to_list(single_dict: dict) -> list:
    """Used for switching the dict shit to csv/list shit"""
    list_shit: list = []
    for _, contents in single_dict.items():
        list_shit.append(contents)
    return list_shit


def event_dict_to_list(big_dict: dict) -> list:
    """Gib meh event result dict"""
    big_list: list = [[x for x in big_dict[0]]]
    for _, contents in big_dict.items():
        big_list.append(__single_dict_to_list(contents))
    return big_list


def list_to_dict(dict_list: list[dict]) -> dict:
    """ Takes a list of dicts and puts them into an index dict """
    new_dict: dict = {}
    for index, contents in enumerate(dict_list):
        new_dict[index] = contents
    return new_dict


def collate_index(page_data: dict) -> dict:
    """ Combines the data values """
    data_list: list = []
    for stuff in page_data:
        data_list.append(page_data[stuff]['data'])
    final_list: list = []
    for dicty_bois in data_list:
        for single_dict in dicty_bois:
            final_list.append(single_dict)
    switch_to_dict = list_to_dict(final_list)
    return switch_to_dict


def convert_to_py(js_vars: str) -> dict:
    """ I really don't care at this stage """
    py_dict = eval_js(js_vars.lstrip('[<script type="application/javascript">').rstrip('</script>]'))
    return py_dict


def resolve_to_ip(url: str) -> str:
    """ Returns IP address of the subdomain """
    return socket.gethostbyname(url)


def pull_tables(page_content: Response) -> list:
    """ Returns a dict with details of all the tables within it """
    debug("pull_tables called")
    soup_parse = BeautifulSoup(page_content.text, "html.parser")
    table_list: list = []
    formatted_table: list = []
    for tables in soup_parse.find_all("table"):
        table_list.append(tables)
    if len(table_list) > 1:
        info(f"multiple tables: {len(table_list)}")
        formatted_table = extract_table(table_list, multiple_tables=True)
    elif len(table_list) == 1:
        info("single table")
        formatted_table = extract_table(table_list[0])
    return formatted_table


def extract_table(table, multiple_tables=False) -> list:
    """ Extracts the HTML table """
    debug("extract_table called")
    parsed_table: list = []
    if not multiple_tables:
        parsed_table.append(strip_table_headers(table))
        parsed_table.append(strip_table_body(table))
    elif multiple_tables:
        # Strip the headers from the first table only
        parsed_table.append(strip_table_headers(table[0]))
        for rows in table:
            parsed_table.append(strip_table_body(rows))
    return flatten_list(parsed_table)


def flatten_list(nested_list: list) -> list:
    """ Removes any nested lists, so we have one big list for each row """
    debug("flatten_list called")
    start_list = [x for x in nested_list if x != []]  # Drop any empty lists initially
    return recursive_anti_nester(start_list)


def recursive_anti_nester(nested_list: list) -> list:
    """ IT'S YA BOI, RECURSION! """
    debug("recursive_anti_nester called")

    class AntiNester(list):
        """ Being simultaneously lazy and extra """

        def __init__(self):
            super().__init__()
            self.flat_list: list = []

        def flatten_that_shit(self, big_list: list) -> None:
            """ RUN IT """
            for line_count, lines in enumerate(big_list):
                if not any(isinstance(index, list) for index in lines):
                    self.flat_list.append(lines)
                    # big_list.pop(line_count)
                else:
                    self.flatten_that_shit(big_list[line_count])

    magic_thing = AntiNester()
    magic_thing.flatten_that_shit(nested_list)
    return magic_thing.flat_list


def strip_table_headers(table) -> list:
    """ Strips the table headers """
    debug("strip_table_headers called")
    headers = []
    for tbl_hdr in table.find("tr").find_all("th"):
        headers.append(tbl_hdr.text.strip())
    return headers


def strip_table_body(table):
    """Given a table, returns all its rows"""
    debug("strip_table_body called")
    rows = []
    for tbl_row in table.find_all("tr")[1:]:
        cells = []
        tds = tbl_row.find_all("td")
        if len(tds) == 0:
            ths = tbl_row.find_all("th")
            for tbl_hdr in ths:
                cells.append(tbl_hdr.text.strip())
        else:
            for tbl_dat in tds:
                links = tbl_dat.find_all(href=True)
                if len(links) == 1:
                    cells.append(strip_report_id(links[0]['href']))
                elif len(tbl_dat.find_all('i')) == 1:
                    strip_it = str(tbl_dat.find_all('i'))
                    if "data-id-resource" in strip_it:
                        re_search = re.search(r'\d+', strip_it)
                        cells.append(re_search.group())
                    else:
                        cells.append(strip_it)
                else:
                    cells.append(tbl_dat.text.strip())
        rows.append(cells)
    return rows


def strip_report_id(url: str) -> str:
    """ This could probably be done a bit neater but IDC currently """
    url_endpoint = LegacyEndPoint.START_LIST.value
    if url_endpoint in url:
        extracted_url = re.search(url_endpoint, url)
        return url[extracted_url.regs[0][1]::]
    return url


def convert_to_json(big_list: list) -> dict:
    """ Takes the data and converts it to a dict so its easier to handle on a webpage FE """
    col_heads = big_list[0]
    dict_to_return = {}
    for index, line in enumerate(big_list[1::]):
        dict_to_return[index] = _insert_json_contents(col_heads, line)
    return dict_to_return


def _insert_json_contents(headers: list, contents: list) -> dict:
    """ Uses the headers supplied to generate the contents of the dict returned"""
    small_dict = {}
    for index, details in enumerate(headers):
        small_dict.update({details: contents[index]})
    return small_dict


def dump_to_csv(filename: str, data_list: list):
    """ This is really lazy and i'll probably remove this """
    full_filename = f"{filename}.csv"
    with open(full_filename, "w", encoding='utf-8') as csv_file:
        writer = csv.writer(csv_file, lineterminator='\n')
        for line in data_list:
            writer.writerow(line)
    return full_filename



""" Moving the request dicts here as dataclasses """
from dataclasses import dataclass


@dataclass
class FilterByYear:
    """ Allows a neater call for any year filtering """
    year: int

    def __init__(self, year: int):
        self.year = year
        self.start_date = f"{year}-01-01"
        self.end_date = f"{year}-12-31"

    def as_dict(self) -> dict:
        """ Returns a dict """
        return {"date_range_start": self.start_date, "date_range_end": self.end_date}


@dataclass
class RequestHeaders:
    """ RequestHeaders that are required for all API calls """
    api_token: str
    auth_url: str

    def __init__(self, api_token: str, auth_url: str):
        self.api_token = api_token
        self.auth_url = auth_url

    def as_dict(self) -> dict:
        """ Returns a dict """
        headers = {"X-API-TOKEN": self.api_token,
                   "authority": self.auth_url,
                   "accept": "application/json",
                   "Content-Type": "application/json"}
        return headers


@dataclass
class PayloadGenerator:
    """ Kinda self-explanatory """
    year: int
    weight_class: int

    def __init__(self, year: int = None, weight_class: int = None):
        self.year: int = year
        self.weight_class: int = weight_class
        self.payload: dict = {}

    def generate(self) -> dict:
        """ Returns a dict """



""" Busy backend shit """
import logging
import requests
from typing import Union, Optional

from urllib.parse import urljoin
from bs4 import BeautifulSoup

from .pages_enum import EndPoint, LegacyEndPoint
from .helpers import pull_tables, convert_to_json, convert_to_py, collate_index, event_dict_to_list


class SportEightyHTTP:
    """ Contains all the big annoying functions so the main API file is nice and neat """

    def __init__(self, domain: str, return_dict: bool = True, debug_lvl: logging = logging.WARNING):
        self.http_session = requests.Session()
        self.domain: str = domain
        self.return_dict: bool = return_dict
        logging.basicConfig(level=debug_lvl)
        self.domain_env = self.pull_domain_env()
        self.standard_headers = self.load_standard_headers()

    def load_standard_headers(self):
        """ Standard header payload for each API request """
        headers = {"X-API-TOKEN": self.domain_env['SERVICES_API_PUBLIC_KEY'],
                   "authority": self.domain_env['RANKINGS_DOMAIN_URL'],
                   "accept": "application/json",
                   "Content-Type": "application/json"}
        return headers

    def app_data(self):
        """ Fetches OpenAPI server details """
        get_page = self.http_session.get(self.domain_env['CORE_SERVICE_API_URL'])
        return get_page.json()

    def pull_domain_env(self) -> dict:
        """ On both BWL and USAW sites, there is a JS dict needed for the API calls to work """
        get_page = requests.get(urljoin(self.domain, EndPoint.INDEX_PAGE.value))
        soup = BeautifulSoup(get_page.content, "html.parser")
        scripts_in_page = soup.find_all('script')
        js_extract = []
        for js_section in scripts_in_page:
            if "application/javascript" in js_section.attrs.values():
                js_extract.append(js_section)
        if len(js_extract) == 1:
            return convert_to_py(str(js_extract))

    def test_token(self, token: str):
        api_url = urljoin(self.domain_env['RANKINGS_DOMAIN_URL'], EndPoint.RANKINGS_INDEX.value)
        get_page = self.http_session.get(api_url, headers={"X-API-TOKEN": token})
        if get_page.status_code == 200:
            return True

    def test_core_api(self):
        api_url = "https://core.sport80.com/api/docs"
        get_page = self.http_session.get(api_url, headers=self.standard_headers)
        if get_page.ok:
            return get_page.json()

    def get_weight_class(self):
        api_url = urljoin(self.domain_env['RANKINGS_DOMAIN_URL'], EndPoint.RANKINGS_DATA.value)
        get_page = self.http_session.get(api_url, headers=self.standard_headers)
        if get_page.ok:
            return get_page.json()

    def get_ranking_index(self):
        """ Working """
        api_url = urljoin(self.domain_env['RANKINGS_DOMAIN_URL'], EndPoint.RANKINGS_INDEX.value)
        get_page = self.http_session.get(api_url, headers=self.standard_headers)
        if get_page.ok:
            return get_page.json()['cards']

    def _get_rankings_table(self, category):
        """ Simple GET call for the ranking category specified """
        api_url = urljoin(self.domain_env['RANKINGS_DOMAIN_URL'], EndPoint.rankings_url(category))
        get_page = self.http_session.get(api_url, headers=self.standard_headers)
        return get_page.json()

    def quick_ranking_search(self):
        """ Cycles through the API endpoint available for rankings table """
        start_cat_int = 1
        start_cat = self._get_rankings_table(start_cat_int)
        err_msg = "An error occurred"
        available_end_points = {}
        while err_msg not in start_cat['title']:
            available_end_points.update({start_cat['title']: start_cat['data_url']})
            start_cat_int += 1
            start_cat = self._get_rankings_table(start_cat_int)
        return available_end_points

    def get_rankings_table(self, category, a_date, z_date, wt_class):
        api_url = urljoin(self.domain_env['RANKINGS_DOMAIN_URL'], EndPoint.rankings_url(category))
        payload = {"date_range_start": a_date, "date_range_end": z_date, "weight_class": wt_class}
        get_page = self.http_session.get(api_url, headers=self.standard_headers)
        return get_page.json()

    def get_rankings(self, wt_class: int, a_date: str, z_date: str):
        api_url = urljoin(self.domain_env['RANKINGS_DOMAIN_URL'], EndPoint.ALL_RANKINGS.value)
        payload = {"date_range_start": a_date, "date_range_end": z_date, "weight_class": wt_class}
        get_page = self.http_session.post(api_url, headers=self.standard_headers, json=payload)
        if get_page.ok:
            return get_page.json()

    def get_event_index(self, year: int) -> dict:
        """ Fetches the event index per year """
        api_url = urljoin(self.domain_env['RANKINGS_DOMAIN_URL'], EndPoint.EVENT_INDEX.value)
        payload = {"date_range_start": f"{year}-01-01", "date_range_end": f"{year}-12-31"}
        get_page = self.http_session.post(api_url, headers=self.standard_headers, json=payload)
        if get_page.ok:
            page_data = self.__collate_results(get_page.json(), payload)
            collated_index = collate_index(page_data)
            return collated_index

    def get_event_results(self, event_dict: dict):
        """ Uses the integer that follows the event url API """
        # todo: below line needs serious refactoring
        event_id: str = event_dict['action'][0]['route'].split('/')[-1]
        api_url = urljoin(self.domain_env['RANKINGS_DOMAIN_URL'], EndPoint.event_results_url(event_id))
        get_page = self.http_session.post(api_url, headers=self.standard_headers)
        collated_pages = self.__collate_results(get_page.json())
        combined_data = collate_index(collated_pages)
        if get_page.ok and self.return_dict:
            return combined_data
        if get_page.ok and not self.return_dict:
            return event_dict_to_list(combined_data)

    def __collate_results(self, page_one: dict, payload: Optional[dict] = None) -> dict:
        """ Cycles through the passed dict and checks for a URL """
        all_pages = {0: page_one}
        current_page = page_one
        index = 1
        while current_page['next_page_url']:
            all_pages[index] = current_page = self.__next_page(current_page['next_page_url'], payload)
            index = index + 1
        return all_pages

    def __next_page(self, next_url: str, payload: dict) -> dict:
        """ Designed around the events dict """
        get_page = self.http_session.post(next_url, headers=self.standard_headers, json=payload)
        if get_page.ok:
            return get_page.json()

    def get_lifter_data(self, lifter_id):
        """ Historical performance of a lifter  """
        api_url = urljoin(self.domain_env['RANKINGS_DOMAIN_URL'], EndPoint.lifter_url(lifter_id))
        get_page = self.http_session.post(api_url, headers=self.standard_headers)
        if get_page.ok:
            return self.__collate_results(get_page.json())

    # LEGACY CODE THAT STILL WORKS
    def get_upcoming_events(self) -> Union[list, dict]:
        """ Returns the upcoming events list """
        logging.info("get_upcoming_events called")
        api_url = urljoin(self.domain, LegacyEndPoint.UPCOMING_EVENTS.value)
        get_page = self.http_session.get(api_url)
        upcoming_events = pull_tables(get_page)
        if self.return_dict:
            return convert_to_json(upcoming_events)
        return upcoming_events

    def get_start_list(self, event_id: str) -> Union[list, dict]:
        """ Returns a specific upcoming events start list """
        logging.info("get_start_list called")
        api_url = urljoin(self.domain, LegacyEndPoint.START_LIST.value + event_id)
        get_page = self.http_session.get(api_url)
        start_list = pull_tables(get_page)
        if self.return_dict:
            return convert_to_json(start_list)
        return start_list



""" Setup file for PyPi """
from setuptools import setup

setup(
    name="sport80",
    version="2.2.2",
    description="Python API interface for the Sport80 sites",
    long_description='Intentionally empty',
    url="https://github.com/euanwm/sport80_api",
    author="Euan Meston",
    author_email="euanmeston@gmail.com",
    license="BSD",
    install_requires=["requests",
                      "beautifulsoup4",
                      "js2py"],
    classifiers=["Programming Language :: Python :: 3.8",
                 "Programming Language :: Python :: 3.9"],
    python_requires='>=3.8'
)
